#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""This module provides a set of dialog based widgets"""

__all__ = ["TaurusMessageBox", "protectTaurusMessageBox",
           "ProtectTaurusMessageBox", "TaurusExceptHookMessageBox"]

__docformat__ = 'restructuredtext'

import sys

from taurus.external.qt import Qt

from taurus.core.util.excepthook import BaseExceptHook
from taurus.core.util.log import LogExceptHook
from taurus.core.util.wrap import wraps


class TaurusMessageBox(Qt.QDialog):
    """A panel intended to display a taurus error.
    Example::

        dev = taurus.Device("sys/tg_test/1")
        try:
            print dev.read_attribute("throw_exception")
        except PyTango.DevFailed, df:
            msgbox = TaurusMessageBox()
            msgbox.show()

    You can show the error outside the exception handling code. If you do this,
    you should keep a record of the exception information as given by
    :func:`sys.exc_info`::

        dev = taurus.Device("sys/tg_test/1")
        exc_info = None
        try:
            print dev.read_attribute("throw_exception")
        except PyTango.DevFailed, df:
            exc_info = sys.exc_info()

        if exc_info:
            msgbox = TaurusMessageBox(*exc_info)
            msgbox.show()"""

    def __init__(self, err_type=None, err_value=None, err_traceback=None,
                 parent=None, designMode=False):
        Qt.QDialog.__init__(self, parent)
        layout = Qt.QVBoxLayout()
        self.setLayout(layout)
        import taurus.qt.qtgui.panel
        MsgPanel = taurus.qt.qtgui.panel.TaurusMessagePanel
        self._panel = MsgPanel(err_type, err_value, err_traceback,
                               self, designMode)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(self._panel)
        self.panel().buttonBox().accepted.connect(self.accept)
        self._panel.toggledDetails.connect(self._onShowDetails)

    def _onShowDetails(self, show):
        self.adjustSize()

    def panel(self):
        """Returns the :class:`taurus.qt.qtgui.panel.TaurusMessagePanel`.

        :return: the internal panel
        :rtype: taurus.qt.qtgui.panel.TaurusMessagePanel"""
        return self._panel

    def addButton(self, button, role=Qt.QDialogButtonBox.ActionRole):
        """Adds the given button with the given to the button box

        :param button: the button to be added
        :type button: PyQt4.QtGui.QPushButton
        :param role: button role
        :type role: PyQt4.Qt.QDialogButtonBox.ButtonRole"""
        self.panel().addButton(button, role)

    def setIconPixmap(self, pixmap):
        """Sets the icon to the dialog

        :param pixmap: the icon pixmap
        :type pixmap: PyQt4.Qt.QPixmap"""
        self._panel.setIconPixmap(pixmap)

    def setText(self, text):
        """Sets the text of the dialog

        :param text: the new text
        :type text: str"""
        self._panel.setText(text)

    def getText(self):
        """Returns the current text of this panel

        :return: the text for this panel
        :rtype: str"""
        return self._panel.getText()

    def setDetailedText(self, text):
        """Sets the detailed text of the dialog

        :param text: the new text
        :type text: str"""
        self._panel.setDetailedText(text)

    def setError(self, err_type=None, err_value=None, err_traceback=None):
        """Sets the exception object.
        Example usage::

            dev = taurus.Device("sys/tg_test/1")
            exc_info = None
            msgbox = TaurusMessageBox()
            try:
                print dev.read_attribute("throw_exception")
            except PyTango.DevFailed, df:
                exc_info = sys.exc_info()

            if exc_info:
                msgbox.setError(*exc_info)
                msgbox.show()

        :param err_type: the exception type of the exception being handled
                         (a class object)
        :type error: class object
        :param err_value: exception object
        :type err_value: object
        :param err_traceback: a traceback object which encapsulates the call
                              stack at the point where the exception originally
                              occurred
        :type err_traceback: TracebackType"""
        self._panel.setError(err_type, err_value, err_traceback)


_PROTECT_MESSAGE_BOX = None


def _ProtectMessageBox(err_type=None, err_value=None, err_traceback=None,
                       parent=None):
    global _PROTECT_MESSAGE_BOX
    box = _PROTECT_MESSAGE_BOX
    if box is None:
        _PROTECT_MESSAGE_BOX = box = TaurusMessageBox(err_type=err_type,
                                                      err_value=err_value, err_traceback=err_traceback, parent=parent)
        box.panel().setCheckBoxVisible(True)
    else:
        box.setError(err_type=err_type, err_value=err_value,
                     err_traceback=err_traceback)
    if box.panel().checkBoxState() == Qt.Qt.Checked:
        return
    return box


def protectTaurusMessageBox(fn):
    """The idea of this function is to be used as a decorator on any method
    you which to protect against exceptions. The handle of the exception is to
    display a :class:`TaurusMessageBox` with the exception information.
    Example::

        @protectTaurusMessgeBox
        def turnBeamOn(device_name):
            d = taurus.Device(device_name)
            d.TurnOn()
    """

    @wraps(fn)
    def wrapped(*args, **kwargs):
        try:
            return fn(*args, **kwargs)
        except:
            msgbox = _ProtectMessageBox(*sys.exc_info())
            if msgbox is None:
                return
            msgbox.exec_()
    return wrapped


class ProtectTaurusMessageBox(object):
    """The idea of this class is to be used as a decorator on any method
    you which to protect against exceptions. The handle of the exception is to
    display a :class:`TaurusMessageBox` with the exception information. The
    optional parameter title gives the window bar a customized title. The
    optional parameter msg allows you to give a customized message in the
    dialog. Example::

        @ProtectTaurusMessgeBox(title="Error trying to turn the beam on")
        def turnBeamOn(device_name):
            d = taurus.Device(device_name)
            d.TurnOn()
    """

    def __init__(self, title=None, msg=None):
        self._title = title
        self._msg = msg

    def __call__(self, fn):
        @wraps(fn)
        def wrapped(*args, **kwargs):
            try:
                return fn(*args, **kwargs)
            except:
                msgbox = _ProtectMessageBox(*sys.exc_info())
                if msgbox is None:
                    return
                if self._title is not None:
                    msgbox.setWindowTitle(self._title)
                if self._msg is not None:
                    msgbox.setText(self._msg)
                msgbox.exec_()
        return wrapped


class TaurusExceptHookMessageBox(BaseExceptHook):
    """A callable class that acts as an excepthook that displays an unhandled
    exception in a :class:`~taurus.qt.qtgui.dialog.TaurusMessageBox`.

    :param hook_to: callable excepthook that will be called at the end of
                    this hook handling [default: None]
    :type hook_to: callable
    :param title: message box title [default: None meaning use exception value]
    :type name: str
    :param msg: message box text [default: None meaning use exception]"""

    MSG_BOX = None

    def __init__(self, hook_to=None, title=None, msg=None):
        BaseExceptHook.__init__(self, hook_to=hook_to)
        self._title = title
        self._msg = msg

    def _getMessageBox(self, err_type=None, err_value=None, err_traceback=None,
                       parent=None):
        box = self.__class__.MSG_BOX

        if box is None:
            self.__class__.MSG_BOX = box = TaurusMessageBox(err_type=err_type,
                                                            err_value=err_value, err_traceback=err_traceback, parent=parent)
            box.panel().setCheckBoxVisible(True)
        else:
            box.setError(err_type=err_type, err_value=err_value,
                         err_traceback=err_traceback)
        if box.panel().checkBoxState() == Qt.Qt.Checked:
            return
        return box

    def report(self, *exc_info):
        app = Qt.QApplication.instance()
        if app is None:
            import taurus.core.util
            LogExceptHook().report(*exc_info)
            return
        msgbox = self._getMessageBox(*exc_info)
        if msgbox is None:
            return
        if self._title is not None:
            msgbox.setWindowTitle(self._title)
        if self._msg is not None:
            msgbox.setText(self._msg)
        msgbox.exec_()


class DemoException(Exception):
    pass


def s1():
    return s2()


def s2():
    return s3()


def s3():
    raise DemoException("A demo exception occurred")


def py_exc():
    try:
        s1()
    except:
        msgbox = TaurusMessageBox(*sys.exc_info())
        msgbox.exec_()


def tg_exc():
    import PyTango
    try:
        PyTango.Except.throw_exception('TangoException',
                                       'A simple tango exception', 'right here')
    except PyTango.DevFailed:
        msgbox = TaurusMessageBox(*sys.exc_info())
        msgbox.exec_()


def tg_serv_exc():
    import PyTango
    import taurus
    dev = taurus.Device("sys/tg_test/1")
    try:
        dev.read_attribute("throw_exception")
    except PyTango.DevFailed:
        msgbox = TaurusMessageBox(*sys.exc_info())
        msgbox.exec_()
    except:
        msgbox = TaurusMessageBox(*sys.exc_info())
        msgbox.exec_()


def py_tg_serv_exc():
    import PyTango
    try:
        PyTango.Except.throw_exception('TangoException',
                                       'A simple tango exception', 'right here')
    except PyTango.DevFailed, df1:
        try:
            import traceback
            import StringIO
            origin = StringIO.StringIO()
            traceback.print_stack(file=origin)
            origin.seek(0)
            origin = origin.read()
            PyTango.Except.re_throw_exception(df1, 'PyDs_Exception',
                                              'DevFailed: A simple tango '
                                              'exception', origin)
        except PyTango.DevFailed:
            msgbox = TaurusMessageBox(*sys.exc_info())
            msgbox.exec_()


def demo():
    """Message dialog"""
    panel = Qt.QWidget()
    layout = Qt.QVBoxLayout()
    panel.setLayout(layout)

    m1 = Qt.QPushButton("Python exception")
    layout.addWidget(m1)
    m1.clicked.connect(py_exc)
    m2 = Qt.QPushButton("Tango exception")
    layout.addWidget(m2)
    m2.clicked.connect(tg_exc)
    layout.addWidget(m2)
    m3 = Qt.QPushButton("Tango server exception")
    layout.addWidget(m3)
    m3.clicked.connect(tg_serv_exc)
    layout.addWidget(m3)
    m4 = Qt.QPushButton("Python tango server exception")
    layout.addWidget(m4)
    m4.clicked.connect(py_tg_serv_exc)
    layout.addWidget(m4)

    panel.show()
    return panel


def main():
    import taurus.qt.qtgui.application

    Application = taurus.qt.qtgui.application.TaurusApplication

    app = Application.instance()
    owns_app = app is None

    if owns_app:
        app = Qt.QApplication([])
        app.setApplicationName("Taurus message demo")
        app.setApplicationVersion("1.0")

    w = demo()

    if owns_app:
        sys.exit(app.exec_())
    else:
        return w

if __name__ == "__main__":
    main()



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""
resfactory.py:
"""

import os
import imp
import operator
import types

from taurus.core.taurushelper import Manager
from taurus.core.util.singleton import Singleton
from taurus.core.util.log import Logger
from taurus.core.taurusfactory import TaurusFactory
from taurus.core.taurusexception import TaurusException


class ResourcesFactory(Singleton, TaurusFactory, Logger):
    """A Singleton class designed to provide Simulation related objects."""

    #: the list of schemes that this factory supports. For this factory: 'res'
    #: and 'resources' are the supported schemes
    schemes = ("res", "resource",)

    #: the default resource file name
    DftResourceName = 'taurus_resources.py'

    #: priority for the default resource
    DftResourcePriority = 10

    def __init__(self):
        """ Initialization. Nothing to be done here for now."""
        pass

    def init(self, *args, **kwargs):
        """Singleton instance initialization.
           **For internal usage only**"""
        name = self.__class__.__name__
        self.call__init__(Logger, name)
        self.call__init__(TaurusFactory)
        self.clear()

    def clear(self):
        self._resource_map = {}
        self._resource_priority = {}
        self._resource_priority_keys = []
        self._resource_count = 0

    def reloadResource(self, obj=None, priority=1, name=None):
        """(Re)Loads the given resource.

           :param obj: (dict or file or None) the resource object. Default is
                       None meaning in will (re)load the default resource:
                       taurus_resources.py from the application directory
           :param priority: (int) the resource priority. Default is 1 meaning
                            maximum priority
           :param name: (str) an optional name to give to the resource

           :return: (dict) a dictionary version of the given resource object
        """
        if priority < 1:
            raise ValueError('priority must be >=1')
        if operator.isMappingType(obj):
            name = name or 'DICT%02d' % priority
        elif type(obj) in types.StringTypes or obj is None:
            name, mod = self.__reloadResource(obj)
            obj = {}
            for k, v in mod.__dict__.items():
                if not k.startswith('_') and isinstance(v, basestring):
                    obj[k] = v
        else:
            raise TypeError

        if self._resource_map.get(name) is None:
            self._resource_count += 1
        self._resource_map[name] = obj
        if self._resource_count == 1:
            self._first_resource = obj

        pl = self._resource_priority.get(priority)
        if pl is None:
            self._resource_priority[priority] = pl = []
        pl.append(name)
        self._resource_priority_keys = self._resource_priority.keys()
        self._resource_priority_keys.sort()
        return obj

    loadResource = reloadResource
    loadResource.__doc__ = reloadResource.__doc__

    def __reloadResource(self, name=None):
        path = os.path.curdir
        if name is None:
            file_name = ResourcesFactory.DftResourceName
        else:
            path, file_name = os.path.split(name)
            if not path:
                path = os.path.curdir
        path = os.path.abspath(path)

        full_name = os.path.join(path, file_name)

        if not os.path.isfile(full_name):
            raise ImportError

        module_name, ext = os.path.splitext(file_name)

        m, file_ = None, None
        try:
            file_, pathname, desc = imp.find_module(module_name, [path])
            self.info("(re)loading resource %s", pathname)
            m = imp.load_module(module_name, file_, pathname, desc)
            if file_:
                file_.close()
        except Exception, e:
            if file_:
                file_.close()
            raise e

        if m is None:
            self.warning("failed to (re)load resource %s" % module_name)
            raise ImportError

        return full_name, m

    def getValue(self, key):
        """Returns the value for a given key

           :param key: (str) a key

           :return: (str) the value for the given key
        """
        if self._resource_count == 0:
            try:
                self.reloadResource(priority=self.DftResourcePriority)
            except:
                return None

        # optimization: many applications contain only one resource: in that
        # case avoid the loop
        if self._resource_count == 1:
            return self._first_resource.get(key, None)

        for p in self._resource_priority_keys:
            for resource_name in self._resource_priority[p]:
                resource = self._resource_map[resource_name]
                try:
                    return resource[key]
                except:
                    pass

    def findObjectClass(self, absolute_name):
        """
        Obtain the class object corresponding to the given name.

        :param absolute_name: (str) the object absolute name string

        :return: (taurus.core.taurusmodel.TaurusModel or None) the class
                 for the model object mapped by absolute_name, or None if
                 absolute_name is invalid.
        """
        validators = (self.getAttributeNameValidator(),
                      self.getDeviceNameValidator(),
                      self.getAuthorityNameValidator())

        for v in validators:
            try:
                value = self.getValue(
                    v.getUriGroups(absolute_name)['_resname'])
                return Manager().findObjectClass(value)
            except:
                pass
        return None

    def getAuthority(self, name=None):
        """
        Obtain the authority model object referenced by name.

        :param name: (str) name

        :return: (taurus.core.taurusauthority.TaurusAuthority) authority object
        :raise: (taurus.core.taurusexception.TaurusException) if name is invalid
        """
        groups = self.getAuthorityNameValidator().getUriGroups(name)
        if groups is None:
            raise TaurusException('Invalid name "%s"' % name)
        res_name = groups['_resname']
        value = self.getValue(res_name)
        return Manager().getAuthority(value)

    def getDevice(self, name):
        """
        Obtain the device model object referenced by name.

        :param name: (str) name

        :return: (taurus.core.taurusdevice.TaurusDevice) device object
        :raise: (taurus.core.taurusexception.TaurusException) if name is invalid
        """
        groups = self.getDeviceNameValidator().getUriGroups(name)
        if groups is None:
            raise TaurusException('Invalid name "%s"' % name)
        res_name = groups['_resname']
        value = self.getValue(res_name)
        return Manager().getDevice(value)

    def getAttribute(self, name):
        """
        Obtain the attribute model object referenced by name.

        :param name: (str) name

        :return: (taurus.core.taurusattribute.TaurusAttribute) attribute object
        :raise: (taurus.core.taurusexception.TaurusException) if name is invalid
        """
        groups = self.getAttributeNameValidator().getUriGroups(name)
        if groups is None:
            raise TaurusException('Invalid name "%s"' % name)
        res_name = groups['_resname']
        value = self.getValue(res_name)
        return Manager().getAttribute(value)

    def getAuthorityNameValidator(self):
        """Return ResourceAuthorityNameValidator"""
        import resvalidator
        return resvalidator.ResourceAuthorityNameValidator()

    def getDeviceNameValidator(self):
        """Return ResourceDeviceNameValidator"""
        import resvalidator
        return resvalidator.ResourceDeviceNameValidator()

    def getAttributeNameValidator(self):
        """Return ResourceAttributeNameValidator"""
        import resvalidator
        return resvalidator.ResourceAttributeNameValidator()



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

""" A minimal application using the Qt console-style IPython frontend.

This is not a complete console app, as subprocess will not be able to receive
input, there is no real readline support, among other limitations.
"""

__all__ = ["TaurusConsoleFactory"]

__docformat__ = 'restructuredtext'

from taurus.core.util.singleton import Singleton
from taurus.external.qt import Qt

from taurusconsolewidget import TaurusConsoleWidget
from taurusconsolewindow import TaurusConsoleWindow
from taurusconsoleapplication import TaurusConsoleApplication

import taurusconsoleextensions
try:
    from IPython.qt.kernelmanager import QtKernelManager
except ImportError:
    from IPython.frontend.qt.kernelmanager import QtKernelManager


class TaurusConsoleFactory(Singleton):

    ipython_application_class = TaurusConsoleApplication
    widget_factory_class = TaurusConsoleWidget
    kernel_manager_class = QtKernelManager

    def init(self, *args, **kwargs):
        self.ipython_application = None
        self.ipython_args = kwargs.pop('ipython_args', [])

    def get_ipython_application(self):
        app = self.ipython_application
        if app is None:
            self.ipython_application = app = self.ipython_application_class()
            app.kernel_manager_class = self.new_kernel_manager
            app.initialize(argv=self.ipython_args)
            app.widget_factory = self.new_frontend_widget
            self.orig_config = app.config.copy()
        return app

    def get_extensions(self):
        import inspect
        ret = {}
        for obj_name in dir(taurusconsoleextensions):
            if obj_name.startswith("_"):
                continue
            obj = getattr(taurusconsoleextensions, obj_name)
            if inspect.isclass(obj):
                if issubclass(obj, taurusconsoleextensions.BaseConsoleExtension):
                    if obj.is_enabled():
                        ret[obj.Name] = obj
        return ret

    def get_extension(self, name):
        import inspect
        ret = {}
        for obj_name in dir(taurusconsoleextensions):
            if obj_name.startswith("_"):
                continue
            obj = getattr(taurusconsoleextensions, obj_name)
            if inspect.isclass(obj):
                if issubclass(obj, taurusconsoleextensions.BaseConsoleExtension):
                    if obj.is_enabled():
                        if obj.Name == name:
                            return obj
                        ret[obj.Name] = obj

    def new_kernel_manager(self, **kwargs):
        return self.kernel_manager_class(**kwargs)

    def new_frontend_widget(self, *args, **kwargs):
        return self.widget_factory_class(*args, **kwargs)

    def new_frontend_slave(self, widget):
        app = self.get_ipython_application()
        new_widget = app.new_frontend_slave(widget)
        return new_widget

    def new_frontend_master(self, name="ipython"):
        app = self.get_ipython_application()
        extension = self.get_extension(name)
        with extension(self):
            return app.new_frontend_master()

    def new_window(self, kernels=None):
        qtapp = Qt.QApplication.instance()
        window = TaurusConsoleWindow(qtapp,
                                     new_frontend_factory=self.new_frontend_master,
                                     slave_frontend_factory=self.new_frontend_slave)
        extensions = self.get_extensions()
        for extension in extensions.values():
            window.register_kernel_extension(extension)
        if kernels is not None:
            for kernel in kernels:
                if isinstance(kernel, tuple):
                    name, label = kernel
                else:
                    name, label = kernel, kernel
                window.create_tab_with_new_frontend(name=name, label=label)
        return window


def main(argv=None):
    import taurus.core.util.argparse
    import taurus.qt.qtgui.application

    targp = taurus.core.util.argparse

    if argv is None:
        import sys
        argv = sys.argv

    parser = targp.get_taurus_parser()
    taurus_args, ipython_args = targp.split_taurus_args(parser, args=argv)

    app = taurus.qt.qtgui.application.TaurusApplication(taurus_args,
                                                        cmd_line_parser=parser)

    console_factory = TaurusConsoleFactory(ipython_args=ipython_args)
    window = console_factory.new_window(kernels=[('ipython', 'IPython')])
    window.create_tab_with_new_frontend(name='tango', label="Tango")
    window.show()
    app.exec_()


if __name__ == '__main__':
    main()



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""
widgetgen.py:
"""

#!/usr/bin/env python

import os
import sys

usage = "usage: %s <widget class name> <widget super class> <output file name> [<qt designer plugin file>]" % os.path.basename(sys.argv[
                                                                                                                               0])

DftModule = "taurus.widget"
DftIconFilename = "label.png"
DftIsContainer = str(False)


def go(class_name, super_class, output_name):

    path = ''
    try:
        path = os.path.dirname(__file__)
    except:
        pass
    input_name = os.path.realpath(path) + '/tauruswidget_template'

    try:
        output = open(output_name, 'w')
    except:
        raise

    try:
        input = open(input_name, 'r')
    except:
        output.close()
        raise

    for s in input:
        o = s.replace('<_SuperClass_>', super_class)
        o = o.replace('<_TaurusClass_>', class_name)
        o = o.replace('<_Module_>', DftModule)
        o = o.replace('<_IconFileName_>', DftIconFilename)
        o = o.replace('<_Container_>', DftIsContainer)
        output.write(o)

    input.close()
    output.close()


def go_with_designer(class_name, super_class, output_name, plugin_output_name):

    go(class_name, super_class, output_name)

    path = ''
    try:
        path = os.path.dirname(__file__)
    except:
        pass
    input_name = os.path.realpath(
        path) + '/tauruswidget_qtdesignerplugin_template'

    try:
        output = open(plugin_output_name, 'w')
    except:
        raise

    try:
        input = open(input_name, 'r')
    except:
        output.close()
        raise

    for s in input:
        o = s.replace('<_SuperClass_>', super_class)
        o = o.replace('<_TaurusClass_>', class_name)
        o = o.replace('<_Module_>', DftModule)
        o = o.replace('<_IconFileName_>', DftIconFilename)
        o = o.replace('<_Container_>', DftIsContainer)
        output.write(o)

    input.close()
    output.close()

if __name__ == "__main__":

    argc = len(sys.argv)
    if argc < 4:
        print usage
    elif argc == 4:
        go(sys.argv[1], sys.argv[2], sys.argv[3])
    else:
        go_with_designer(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])



import sys
from taurus.external.qt import Qt
from taurus.qt.qtgui.display import TaurusLabel

app = Qt.QApplication(sys.argv)
panel = Qt.QWidget()
layout = Qt.QGridLayout()
panel.setLayout(layout)
for y in range(4):
    for x in range(2):
        w = TaurusLabel()
        w.setModel('sys/taurustest/%d/state' % ((y + 1) + 4 * x))
        w.setShowText(False)
        w.setBgRole('state')
        layout.addWidget(w, x, y)
panel.show()

sys.exit(app.exec_())



#!/usr/bin/env python
#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################


"""Utilities for creating generic tests for Taurus widgets"""

import taurus.core
from taurus.external import unittest
from taurus.qt.qtgui.application import TaurusApplication


class BaseWidgetTestCase(object):

    '''
    A base class for tests that need a widget instance

    To use it, simply inherit from BaseWidgetTestCase *and* unittest.TestCase
    and provide the following class members:

      - _klass (typeobject) the widget class to test (mandatory)
      - initargs (list) a list of arguments for the klass init method
                 (default=[])
      - initkwargs (dict) a dict of keyword arguments for the klass init method
                   (default={})
    '''
    _klass = None
    initargs = []
    initkwargs = {}

    def setUp(self):
        """
        Preconditions:

          - A TaurusApplication must be initialized.
          - The widget must be instantiated

        """
        unittest.TestCase.setUp(self)

        from taurus.core.util.log import _DEPRECATION_COUNT
        self._depCounter = _DEPRECATION_COUNT
        self._depCounter.clear()

        app = TaurusApplication.instance()
        if app is None:
            app = TaurusApplication([])
        self._app = app

        if self._klass is not None:
            self._widget = self._klass(*self.initargs, **self.initkwargs)

    def assertMaxDeprecations(self, maximum, msg=None):
        """Assertion method that checks that the number of deprecations issued
        during the current test is equal to or below a given maximum

        :param maximum: (int) maximum number of deprecation warnings allowed
        """
        deps = self._depCounter.getTotal()
        if msg is None:
            msg = ('%d deprecation warnings issued (max=%d):\n%s' %
                   (deps, maximum, self._depCounter.pretty()))
        self.assertTrue(deps <= maximum, msg)


class GenericWidgetTestCase(BaseWidgetTestCase):

    '''a base class for testing common cases of arbitrary Taurus widget classes

    To use it, simply inherit from GenericWidgetTestCase *and* unittest.TestCase
    and provide the following class members:

      - _klass (typeobject) the widget class to test (mandatory)
      - initargs (list) a list of arguments for the klass init method
                 (default=[])
      - initkwargs (dict) a dict of keyword arguments for the klass init method
                   (default={})
      - modelnames (list) a list of model names which the widget should be able
                   to handle (default=[])
    '''
    modelnames = []

    def setUp(self):
        """
        Preconditions:

          - Those from :class:`BaseWidgetTestCase`
          - A list of models corresponding to the modelnames list
            should be created without using the widget being tested
            (e.g. by using taurusManager.findObject()).
            None should be used as a placeholder when a model cannot be created
            for a given modelname.
        """
        # Make sure the basics are taken care of (QApplication, etc)
        BaseWidgetTestCase.setUp(self)

        # construct a list of models corresponding to the test model names
        # provided by the widget
        taurusManager = taurus.core.TaurusManager()
        self._models = []
        for n in self.modelnames:
            if not n:
                # an empty string or None are "valid" modelnames which should
                # lead to a reset in the model
                self._models.append(None)
            else:
                try:
                    # note, an unsupported model name will result in a None,
                    # which will be caught later on
                    model = taurusManager.findObject(n)
                except:
                    model = None
                self._models.append(model)

    def test00_Instantiation(self):
        '''Check that the widget instantiates correctly'''
        self.assertIsInstance(self._widget, self._klass)
        self.assertMaxDeprecations(0)

    def test10_SetModelsSequentially(self):
        '''Check that we can set several models sequentially'''

        for name, model in zip(self.modelnames, self._models):
            self._widget.setModel(name)
            modelobj = self._widget.getModelObj()
            if model is None and name:
                # the modelname is not supported by the manager (we cannot test
                # it apart from the fact that setModel does not raise an
                # exception
                continue
            else:
                self.assertIs(modelobj, model,
                              'failed to set model "%s" for %s' %
                              (name, self._klass.__name__))
        self.assertMaxDeprecations(0)

#    def test10_ModelProperty(self):
#        pass


if __name__ == "__main__":
    from taurus.qt.qtgui.display import TaurusLabel

    class TaurusLabelTest(GenericWidgetTestCase, unittest.TestCase):
        _klass = TaurusLabel
        modelnames = ['sys/tg_test/1/wave', '', 'eval:1', None]

    suite = unittest.defaultTestLoader.loadTestsFromTestCase(TaurusLabelTest)
    unittest.TextTestRunner(descriptions=True, verbosity=2).run(suite)
    # unittest.main()
    # TaurusLabelTest().run()



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

""" A minimal application using the Qt console-style IPython frontend.

This is not a complete console app, as subprocess will not be able to receive
input, there is no real readline support, among other limitations.
"""

__all__ = ["TaurusConsoleApplication"]

__docformat__ = 'restructuredtext'


from taurus.external.qt import Qt

try:
    from IPython.qt.console.qtconsoleapp import IPythonQtConsoleApp
except ImportError:  # for IPython v<1.x
    from IPython.frontend.qt.console.qtconsoleapp import IPythonQtConsoleApp


class TaurusConsoleApplication(IPythonQtConsoleApp):

    name = 'taurusconsole'

    def init_qt_elements(self):
        self.app = Qt.QApplication.instance()
        self.app.icon = Qt.QIcon()

    def init_signal(self):
        pass

    def init_kernel_manager(self):
        # avoid starting a default kernel
        self.kernel_manager = None



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""This module provides a panel to display taurus messages"""

__all__ = ["TaurusMessagePanel", "TaurusMessageErrorHandler",
           "TangoMessageErrorHandler", "MacroServerMessageErrorHandler"]

__docformat__ = 'restructuredtext'

import sys
import traceback
import datetime

try:
    import pygments
    from pygments import highlight
    from pygments.formatters import HtmlFormatter
    from pygments.lexers import PythonTracebackLexer
except:
    pygments = None

# shame on me for importing PyTango! well not so much since this is designed to
# show PyTango exceptions
import PyTango

from taurus.core.util.report import TaurusMessageReportHandler
from taurus.external.qt import Qt
from taurus.qt.qtgui.util.ui import UILoadable


class TaurusMessageErrorHandler(object):
    """This class is designed to handle a generic error into a
    :class:`TaurusMessagePanel`"""

    def __init__(self, msgbox):
        self._msgbox = msgbox
        msgbox.setWindowTitle("Taurus Error")
        self.setError(*msgbox.getError())

    def setError(self, err_type=None, err_value=None, err_traceback=None):
        """Translates the given error object into an HTML string and places it
        in the message panel

        :param error: an error object (typically an exception object)
        :type error: object"""

        msgbox = self._msgbox
        error = "".join(traceback.format_exception_only(err_type, err_value))
        msgbox.setText(error)
        msg = "<html><body><pre>%s</pre></body></html>" % error
        msgbox.setDetailedHtml(msg)

        html_orig = '<html><head><style type="text/css">{style}</style>' \
                    '</head><body>'
        exc_info = "".join(traceback.format_exception(err_type, err_value,
                                                      err_traceback))
        style = ""
        if pygments is not None:
            formatter = HtmlFormatter()
            style = formatter.get_style_defs()
        html = html_orig.format(style=style)
        if pygments is None:
            html += "<pre>%s</pre>" % exc_info
        else:
            formatter = HtmlFormatter()
            html += highlight(exc_info, PythonTracebackLexer(), formatter)
        html += "</body></html>"
        msgbox.setOriginHtml(html)


class TangoMessageErrorHandler(TaurusMessageErrorHandler):
    """This class is designed to handle :class:`PyTango.DevFailed` error into
    a :class:`TaurusMessagePanel`"""

    def setError(self, err_type=None, err_value=None, err_traceback=None):
        """Translates the given error object into an HTML string and places it
        it the message panel

        :param error: an error object (typically an exception object)
        :type error: object"""

        msgbox = self._msgbox
        html_orig = '<html><head><style type="text/css">{style}</style>' \
                    '</head><body>'
        style, formatter = "", None
        if pygments is not None:
            formatter = HtmlFormatter()
            style = formatter.get_style_defs()
        html = html_orig.format(style=style)
        for de in err_value:
            e_html = """<pre>{reason}: {desc}</pre>{origin}<hr>"""
            origin, reason, desc = de.origin, de.reason, de.desc
            if reason.startswith("PyDs_") and pygments is not None:
                origin = highlight(origin, PythonTracebackLexer(), formatter)
            else:
                origin = "<pre>%s</pre>" % origin
            html += e_html.format(desc=desc, origin=origin, reason=reason)
        html += "</body></html>"
        msgbox.setText(err_value[0].desc)
        msgbox.setDetailedHtml(html)

        exc_info = "".join(traceback.format_exception(err_type, err_value,
                                                      err_traceback))
        html = html_orig.format(style=style)
        if pygments is None:
            html += "<pre>%s</pre>" % exc_info
        else:
            html += highlight(exc_info, PythonTracebackLexer(), formatter)
        html += "</body></html>"
        msgbox.setOriginHtml(html)


class MacroServerMessageErrorHandler(TaurusMessageErrorHandler):

    def setError(self, err_type=None, err_value=None, err_traceback=None):
        """Translates the given error object into an HTML string and places it
        in the message panel

        :param error: an error object (typically an exception object)
        :type error: object"""

        msgbox = self._msgbox
        msgbox.setText(err_value)
        msg = "<html><body><pre>%s</pre></body></html>" % err_value
        msgbox.setDetailedHtml(msg)

        html_orig = """<html><head><style type="text/css">{style}</style></head><body>"""
        exc_info = "".join(err_traceback)
        style = ""
        if pygments is not None:
            formatter = HtmlFormatter()
            style = formatter.get_style_defs()
        html = html_orig.format(style=style)
        if pygments is None:
            html += "<pre>%s</pre>" % exc_info
        else:
            formatter = HtmlFormatter()
            html += highlight(exc_info, PythonTracebackLexer(), formatter)
        html += "</body></html>"
        msgbox.setOriginHtml(html)


def is_report_handler(report, abs_file=None):
    """Helper function to determine if a certain python object is a valid
    report handler"""
    import inspect
    if not inspect.isclass(report):
        return False
    if not issubclass(report, TaurusMessageReportHandler):
        return False
    try:
        if inspect.getabsfile(report) != abs_file:
            return False
    except:
        return False
    return True

_REPORT_HANDLERS = None


def get_report_handlers():
    global _REPORT_HANDLERS
    if not _REPORT_HANDLERS is None:
        return _REPORT_HANDLERS

    import os.path
    import functools
    import inspect

    this = os.path.abspath(__file__)
    report_path = os.path.join(os.path.dirname(this), "report")
    sys.path.insert(0, report_path)

    _REPORT_HANDLERS = {}
    try:
        for elem in os.listdir(report_path):
            if not elem[0].isalpha():
                continue
            full_elem = os.path.join(report_path, elem)
            if not os.path.isfile(full_elem):
                continue
            if not elem.endswith('.py'):
                continue
            elem, _ = os.path.splitext(elem)
            _is_report_handler = functools.partial(
                is_report_handler, abs_file=full_elem)
            report_lib = __import__(elem, globals(), locals(), [], -1)
            for name, obj in inspect.getmembers(report_lib, _is_report_handler):
                _REPORT_HANDLERS[name] = obj
    finally:
        sys.path.pop(0)

    return _REPORT_HANDLERS

_REPORT = """\
-- Description -----------------------------------------------------------------
An error occured in '{appName} {appVersion}' on {time}
{text}

-- Details ---------------------------------------------------------------------
{detail}

-- Origin ----------------------------------------------------------------------
{origin}
--------------------------------------------------------------------------------
"""


@UILoadable(with_ui='_ui')
class TaurusMessagePanel(Qt.QWidget):
    """A panel intended to display a taurus error.
    Example::

        dev = taurus.Device("sys/tg_test/1")
        try:
            print dev.read_attribute("throw_exception")
        except PyTango.DevFailed, df:
            msgbox = TaurusMessagePanel()
            msgbox.show()

    You can show the error outside the exception handling code. If you do this,
    you should keep a record of the exception information as given by
    :func:`sys.exc_info`::

        dev = taurus.Device("sys/tg_test/1")
        exc_info = None
        try:
            print dev.read_attribute("throw_exception")
        except PyTango.DevFailed, df:
            exc_info = sys.exc_info()

        if exc_info:
            msgbox = TaurusMessagePanel(*exc_info)
            msgbox.show()"""

    toggledDetails = Qt.pyqtSignal(bool)

    def __init__(self, err_type=None, err_value=None, err_traceback=None, parent=None, designMode=False):
        Qt.QWidget.__init__(self, parent)
        self.loadUi()
        self._exc_info = err_type, err_value, err_traceback

        self._ui._detailsWidget.setVisible(False)
        self._ui._checkBox.setVisible(False)
        self._ui._checkBox.setCheckState(Qt.Qt.Unchecked)
        self._initReportCombo()

        self._ui._showDetailsButton.toggled.connect(self._onShowDetails)
        self._ui._reportComboBox.activated.connect(self._onReportTriggered)

        pixmap = Qt.QIcon.fromTheme("emblem-important").pixmap(48, 48)
        self.setIconPixmap(pixmap)

        if err_value is not None:
            self.setError(*self._exc_info)
        self.adjustSize()

    def _initReportCombo(self):
        report_handlers = get_report_handlers()
        combo = self.reportComboBox()
        for name, report_handler in report_handlers.items():
            name = Qt.QVariant(name)
            combo.addItem(report_handler.Label, name)

    def _onReportTriggered(self, index):
        report_handlers = get_report_handlers()
        combo = self.reportComboBox()
        name = Qt.from_qvariant(combo.itemData(index), str)
        report_handler = report_handlers[name]
        report = report_handler(self)
        app = Qt.QApplication.instance()
        txt = _REPORT.format(appName=app.applicationName(),
                             appVersion=app.applicationVersion(),
                             time=datetime.datetime.now().ctime(),
                             text=self.getText(),
                             detail=self.getDetailedText(),
                             origin=self.getOriginText())
        report.report(txt)

    def _onShowDetails(self, show):
        self._ui._detailsWidget.setVisible(show)
        if show:
            text = "Hide details..."
        else:
            text = "Show details..."
        self._ui._showDetailsButton.setText(text)
        self.adjustSize()
        self.toggledDetails.emit(show)

    def reportComboBox(self):
        return self._ui._reportComboBox

    def checkBox(self):
        """Returns the check box from this panel

        :return: the check box from this panel
        :rtype: PyQt4.Qt.QCheckBox"""
        return self._ui._checkBox

    def checkBoxState(self):
        """Returns the check box state

        :return: the check box state
        :rtype: PyQt4.Qt.CheckState"""
        return self.checkBox().checkState()

    def checkBoxText(self):
        """Returns the check box text

        :return: the check box text
        :rtype: str"""
        return str(self.checkBox().text())

    def setCheckBoxText(self, text):
        """Sets the checkbox text.

        :param text: new checkbox text
        :type text: str"""
        self.checkBox().setText(text)

    def setCheckBoxState(self, state):
        """Sets the checkbox state.

        :param text: new checkbox state
        :type text: PyQt4.Qt.CheckState"""
        self.checkBox().setCheckState(state)

    def setCheckBoxVisible(self, visible):
        """Sets the checkbox visibility.

        :param visible: True makes checkbox visible, False hides it
        :type visible: bool"""
        self.checkBox().setVisible(visible)

    def buttonBox(self):
        """Returns the button box from this panel

        :return: the button box from this panel
        :rtype: PyQt4.Qt.QDialogButtonBox"""
        return self._ui._buttonBox

    def addButton(self, button, role=Qt.QDialogButtonBox.ActionRole):
        """Adds the given button with the given to the button box

        :param button: the button to be added
        :type button: PyQt4.QtGui.QPushButton
        :param role: button role
        :type role: PyQt4.Qt.QDialogButtonBox.ButtonRole"""
        self._ui._buttonBox.addButton(button, role)

    def setIconPixmap(self, pixmap):
        """Sets the icon to the dialog

        :param pixmap: the icon pixmap
        :type pixmap: PyQt4.Qt.QPixmap"""
        self._ui._iconLabel.setPixmap(pixmap)

    def setText(self, text):
        """Sets the text of this panel

        :param text: the new text
        :type text: str"""
        self._ui._textLabel.setText(text)

    def getText(self):
        """Returns the current text of this panel

        :return: the text for this panel
        :rtype: str"""
        return self._ui._textLabel.text()

    def setDetailedText(self, text):
        """Sets the detailed text of the dialog

        :param text: the new text
        :type text: str"""
        self._ui._detailsTextEdit.setPlainText(text)

    def setDetailedHtml(self, html):
        """Sets the detailed HTML of the dialog

        :param html: the new HTML text
        :type html: str"""
        self._ui._detailsTextEdit.setHtml(html)

    def getDetailedText(self):
        """Returns the current detailed text of this panel

        :return: the detailed text for this panel
        :rtype: str"""
        return self._ui._detailsTextEdit.toPlainText()

    def getDetailedHtml(self):
        """Returns the current detailed HTML of this panel

        :return: the detailed HTML for this panel
        :rtype: str"""
        return self._ui._detailsTextEdit.toHtml()

    def setOriginText(self, text):
        """Sets the origin text of the dialog

        :param text: the new text
        :type text: str"""
        self._ui._originTextEdit.setPlainText(text)

    def setOriginHtml(self, html):
        """Sets the origin HTML of the dialog

        :param html: the new HTML text
        :type html: str"""
        self._ui._originTextEdit.setHtml(html)

    def getOriginText(self):
        """Returns the current origin text of this panel

        :return: the origin text for this panel
        :rtype: str"""
        return self._ui._originTextEdit.toPlainText()

    def getOriginHtml(self):
        """Returns the current origin HTML of this panel

        :return: the origin HTML for this panel
        :rtype: str"""
        return self._ui._originTextEdit.toHtml()

    def setError(self, err_type=None, err_value=None, err_traceback=None):
        """Sets the exception object.
        Example usage::

            dev = taurus.Device("sys/tg_test/1")
            exc_info = None
            msgbox = TaurusMessagePanel()
            try:
                print dev.read_attribute("throw_exception")
            except PyTango.DevFailed, df:
                exc_info = sys.exc_info()

            if exc_info:
                msgbox.setError(*exc_info)
                msgbox.show()

        :param err_type: the exception type of the exception being handled
                         (a class object)
        :type error: class object
        :param err_value: exception object
        :type err_value: object
        :param err_traceback: a traceback object which encapsulates the call
                              stack at the point where the exception originally
                              occurred
        :type err_traceback: TracebackType"""
        i = sys.exc_info()
        self._exc_info = [err_type or i[0],
                          err_value or i[1], err_traceback or i[2]]

        handler_klass = self.findErrorHandler(self._exc_info[0])
        handler_klass(self)

    def getError(self):
        """Returns the current exception information of this panel

        :return: the current exception information (same as type as returned by
                 :func:`sys.exc_info`)
        :rtype: tuple<type, value, traceback>"""
        return self._exc_info

    ErrorHandlers = {PyTango.DevFailed: TangoMessageErrorHandler}

    @classmethod
    def registerErrorHandler(klass, err_type, err_handler):
        klass.ErrorHandlers[err_type] = err_handler

    @classmethod
    def findErrorHandler(klass, err_type):
        """Finds the proper error handler class for the given error

        :param err_type: error class
        :type err_type: class object
        :return: a message box error handler
        :rtype: TaurusMessageBoxErrorHandler class object"""

        for exc, h_klass in klass.ErrorHandlers.items():
            if issubclass(err_type, exc):
                return h_klass
        return TaurusMessageErrorHandler


class DemoException(Exception):
    """Just a plain python exception for demo purposes"""
    pass


def s1():
    """Just a function to make the stack more interesting"""
    return s2()


def s2():
    """Just a function to make the stack more interesting"""
    return s3()


def s3():
    """Just a function to make the stack more interesting"""
    raise DemoException("A demo exception occurred")


class QMessageDialog(Qt.QDialog):
    """Helper class for the demo"""

    def __init__(self, msgbox, parent=None):
        Qt.QDialog.__init__(self, parent)
        l = Qt.QVBoxLayout()
        l.setContentsMargins(0, 0, 0, 0)
        l.addWidget(msgbox)
        l.addStretch(1)
        self.setLayout(l)


def py_exc():
    """Shows a python exception in a TaurusMessagePanel"""
    try:
        s1()
    except:
        msgbox = TaurusMessagePanel(*sys.exc_info())
        QMessageDialog(msgbox).exec_()


def tg_exc():
    """Shows a tango exception in a TaurusMessagePanel"""
    try:
        PyTango.Except.throw_exception(
            'TangoException', 'A simple tango exception', 'right here')
    except PyTango.DevFailed:
        msgbox = TaurusMessagePanel(*sys.exc_info())
        QMessageDialog(msgbox).exec_()


def tg_serv_exc():
    """Shows a tango exception from a server in a TaurusMessagePanel"""
    import taurus
    dev = taurus.Device("sys/tg_test/1")
    try:
        dev.read_attribute("throw_exception")
    except PyTango.DevFailed:
        msgbox = TaurusMessagePanel(*sys.exc_info())
        QMessageDialog(msgbox).exec_()
    except:
        msgbox = TaurusMessagePanel(*sys.exc_info())
        QMessageDialog(msgbox).exec_()


def py_tg_serv_exc():
    """Shows a tango exception from a python server in a TaurusMessagePanel"""
    try:
        PyTango.Except.throw_exception(
            'TangoException', 'A simple tango exception', 'right here')
    except PyTango.DevFailed, df1:
        try:
            import traceback
            import StringIO
            origin = StringIO.StringIO()
            traceback.print_stack(file=origin)
            origin.seek(0)
            origin = origin.read()
            PyTango.Except.re_throw_exception(
                df1, 'PyDs_Exception', 'DevFailed: A simple tango exception', origin)
        except PyTango.DevFailed:
            msgbox = TaurusMessagePanel(*sys.exc_info())
            QMessageDialog(msgbox).exec_()


def demo():
    """Message panel"""
    panel = Qt.QWidget()
    layout = Qt.QVBoxLayout()
    panel.setLayout(layout)

    m1 = Qt.QPushButton("Python exception")
    layout.addWidget(m1)
    m1.clicked.connect(py_exc)
    m2 = Qt.QPushButton("Tango exception")
    layout.addWidget(m2)
    m2.clicked.connect(tg_exc)
    layout.addWidget(m2)
    m3 = Qt.QPushButton("Tango server exception")
    layout.addWidget(m3)
    m3.clicked.connect(tg_serv_exc)
    layout.addWidget(m3)
    m4 = Qt.QPushButton("Python tango server exception")
    layout.addWidget(m4)
    m4.clicked.connect(py_tg_serv_exc)
    layout.addWidget(m4)
    return panel


def main():

    import sys
    import taurus.qt.qtgui.application

    Application = taurus.qt.qtgui.application.TaurusApplication

    app = Application.instance()
    owns_app = app is None

    if owns_app:
        app = Qt.QApplication([])
        app.setApplicationName("Taurus message demo")
        app.setApplicationVersion("1.0")

    w = demo()
    w.show()
    if owns_app:
        sys.exit(app.exec_())
    else:
        return w

if __name__ == "__main__":
    main()



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""This module provides a very simple API for starting and killing device
servers

It is not a replacement of the Tango Starter Device Server since this is much
more limited in scope.
"""

__docformat__ = 'restructuredtext'


import os
import time
import subprocess
import PyTango

from taurus.core.util.log import Logger

_log = Logger('Starter')


class Starter(object):
    '''Abstract class for managing (starting, stopping, registering and
    removing) a Tango Device Server.

    Derived classes should provide the methods for starting and stopping a
    device.
    '''

    def __init__(self, ds_name):
        '''
        :param ds_name: (str) Device Server name in the form "server/instance"
        '''
        self.ds_name = ds_name
        self.dserver_name = 'dserver/%s' % ds_name
        try:
            self.dserver = PyTango.DeviceProxy(self.dserver_name)
            self.serverExisted = True
        except PyTango.DevFailed:  # not registered?
            self.dserver = None
            self.serverExisted = False
        self._addedDevices = []

    def hardKill(self):
        raise NotImplementedError('hardKill method is mandatory')

    def terminate(self):
        raise NotImplementedError('terminate method is mandatory')

    def start(self):
        raise NotImplementedError('start method is mandatory')

    def stopDs(self, synch=True, hard_kill=False, wait_seconds=10):
        if hard_kill:
            _log.info('Hard killing server %s...' % self.ds_name)
            self.hardKill()
        else:
            _log.info('Stopping server %s...' % self.ds_name)
            self.terminate()
        if not synch:
            return
        for i in range(wait_seconds):
            _log.debug('Waiting for server %s to get stopped. Iteration: %d' %
                       (self.ds_name, i))
            if self.isRunning():
                time.sleep(1)
            else:
                ##############################################################
                # TODO: this workaround doesn't seem necessary (see isRunning)
                # time.sleep(3)
                ##############################################################
                _log.info('Server %s has been stopped' % self.ds_name)
                return
        _log.warning('Server %s did not stop within %d seconds' %
                     (self.ds_name, wait_seconds))

    def startDs(self, synch=True, wait_seconds=10):
        if self.isRunning():
            _log.warning('Server already running')
            return
        _log.info('Starting server %s...' % self.ds_name)
        self.start()
        if not synch:
            return
        for i in range(wait_seconds):
            _log.debug('Waiting for server %s to get started... %d' %
                       (self.ds_name, i))
            if self.isRunning():
                _log.info('Server %s has been started' % self.ds_name)
                ##############################################################
                # TODO: this workaround doesn't seem necessary (see isRunning)
                # time.sleep(3)
                ##############################################################
                return
            else:
                time.sleep(1)
        _log.warning('Server %s did not start within %d seconds' %
                     (self.ds_name, wait_seconds))

    def addNewDevice(self, device, klass=None):
        """
        Register a device of this server in the DB (register the server if
        not present)
        e.g. to create Starter in an init script::

            addNewDevice('sys/tg_test/foobar', klass='TangoTest')

        :param klass: class name. If None passed, it defaults to the server
                      name (without instance name)
        """
        if device in self._addedDevices:
            _log.warning('%s already added. Skipping' % device)
            return
        if klass is None:
            klass = self.ds_name.split('/')[0]
        # in case the device is already defined, skipping...
        db = PyTango.Database()
        try:
            db.import_device(device)
            _log.warning('%s already exists. Skipping' % device)
            return
        except:
            pass
        # register the device,
        # in case the server did not exist before this will define it
        dev_info = PyTango.DbDevInfo()
        dev_info.name = device
        dev_info.klass = klass
        dev_info.server = self.ds_name
        db.add_device(dev_info)
        # create proxy to dserver
        self.dserver = PyTango.DeviceProxy(self.dserver_name)
        # keep track of added devices
        self._addedDevices.append(device)

    def cleanDb(self, force=False):
        '''removes devices which have been added by :meth:`addNewDevice`
        and then removes the server if it was registered by this starter
        (or, if force is True, it removes the server in any case)

        :param force: (bool) force removing of the Server even if it was
                      not registered within this starter
        '''
        for device in self._addedDevices:
            PyTango.Database().delete_device(device)
            _log.info('Deleted device %s' % device)
        if (self.serverExisted or len(self._addedDevices) == 0) and not force:
            msg = ('%s was not registered by this starter. Not removing. ' +
                   'Use %s.cleanDb(force=True) to force cleanup') % \
                (self.ds_name, self.__class__.__name__)
            _log.warning(msg)
        else:
            self.stopDs(hard_kill=True)
            PyTango.Database().delete_server(self.ds_name)
            _log.info('Deleted Server %s' % self.ds_name)

    def isRunning(self):
        # TODO: In case the sleeps in startDS and stopDS need to be re-added,
        #       we should study another implementation for this method.
        if self.dserver is None:
            return False
        try:
            self.dserver.ping()
        except PyTango.DevFailed:
            return False
        return True


class ProcessStarter(Starter):
    '''A :class:`Starter` which uses subprocess to start and stop a device
    server.
    '''

    def __init__(self, execname, ds_name):
        '''
        :param execname: (str) path to the executable to launch the server
        :param ds_name: (str) Device Server name in the form "server/instance"
        '''
        super(ProcessStarter, self).__init__(ds_name)
        self.ds_instance = ds_name.split('/')[1]
        self.exec_name = os.path.abspath(execname)
        self.process = None

    def start(self):
        dev_null = open(os.devnull, 'wb')
        args = [self.exec_name, self.ds_instance]
        self.process = subprocess.Popen(args, stdout=dev_null, stderr=dev_null)

    def terminate(self):
        if self.process:
            self.process.terminate()
        else:
            _log.warning('Process not started, cannot terminate it.')

    def hardKill(self):
        if self.process:
            self.process.kill()
        else:
            _log.warning('Process not started, cannot terminate it.')


if __name__ == '__main__':

    from taurus.test.resource import getResourcePath
    timeoutExec = getResourcePath('taurus.qt.qtgui.button.test.res', 'Timeout')
    s = ProcessStarter(timeoutExec, 'Timeout/test_removeme')
    devname = 'testing/timeout/temp-1'
    s.addNewDevice(devname, klass='Timeout')
    s.startDs()
    try:
        print 'Is running:', s.isRunning()
        print "ping:", PyTango.DeviceProxy(devname).ping()
    except Exception, e:
        print e
    s.stopDs()
    s.cleanDb(force=False)



#!/usr/bin/env python

#############################################################################
##
# This file is part of Taurus
##
# http://taurus-scada.org
##
# Copyright 2011 CELLS / ALBA Synchrotron, Bellaterra, Spain
##
# Taurus is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
##
# Taurus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
##
# You should have received a copy of the GNU Lesser General Public License
# along with Taurus.  If not, see <http://www.gnu.org/licenses/>.
##
#############################################################################

"""This module provides widgets that display the authority in a tree format"""

__docformat__ = 'restructuredtext'

import os
import re

from taurus.external.qt import Qt

from taurus.core.taurusauthority import TaurusAuthority
from taurus.core.taurusbasetypes import TaurusElementType as ElemType
import taurus.qt.qtgui.base

from taurus.core.tango.tangodatabase import (TangoAttrInfo, TangoDevInfo,
                                             TangoServInfo)

from taurus.qt.qtgui.icon import getElementTypeIcon




class BaseFilter(object):

    def __init__(self, re_expr):
        if len(re_expr) == 0:
            re_expr = ".*"
        else:
            if re_expr[0] != "^":
                re_expr = "^" + re_expr
            if re_expr[-1] != "$":
                re_expr += "$"
        self._re_expr = re.compile(re_expr, re.IGNORECASE)

    def __call__(self, obj):
        return self.filter(obj)


class BaseElementFilter(BaseFilter):

    def __init__(self, re_expr, func=None):
        super(BaseElementFilter, self).__init__(re_expr)
        self._klass = func.im_class
        self._func = func

    def filter(self, obj):
        if not isinstance(obj, self._klass):
            return obj
        v = self._func(obj)
        if self._re_expr.match(v):
            return obj


class DeviceFilter(BaseElementFilter):

    def __init__(self, re_expr, func=TangoDevInfo.name):
        super(DeviceFilter, self).__init__(re_expr, func=func)


class DeviceClassFilter(BaseElementFilter):

    def __init__(self, re_expr, func=TangoDevInfo.name):
        super(DeviceClassFilter, self).__init__(re_expr, func=func)


class ServerFilter(BaseElementFilter):

    def __init__(self, re_expr, func=TangoServInfo.name):
        super(ServerFilter, self).__init__(re_expr, func=func)


class AttributeFilter(BaseElementFilter):

    def __init__(self, re_expr, func=TangoAttrInfo.name):
        super(AttributeFilter, self).__init__(re_expr, func=func)


class KlassFilter(BaseFilter):

    def __init__(self, klass):
        # don't call super on purpose. We don't need/have a regular expression here!
        #super(KlassFilter, self).__init__(re_expr)
        self._klass = klass

    def filter(self, obj):
        if isinstance(obj, self._klass):
            return obj


def getFilter(type, re_expr=None):
    if re_expr is None:
        if type == ElemType.Device:
            return KlassFilter(TangoDevInfo)
        elif type == ElemType.Server:
            return KlassFilter(TangoServInfo)
        elif type == ElemType.DeviceClass:
            return KlassFilter(TangoDevInfo)
        return None

    if type == ElemType.Device:
        return DeviceFilter(re_expr)
    elif type == ElemType.Domain:
        return DeviceFilter(re_expr, TangoDevInfo.domain)
    elif type == ElemType.Family:
        return DeviceFilter(re_expr, TangoDevInfo.family)
    elif type == ElemType.Member:
        return DeviceFilter(re_expr, TangoDevInfo.member)
    elif type == ElemType.Server:
        return ServerFilter(re_expr)
    elif type == ElemType.ServerName:
        return ServerFilter(re_expr, TangoServInfo.serverName)
    elif type == ElemType.ServerInstance:
        return ServerFilter(re_expr, TangoServInfo.serverInstance)
    elif type == ElemType.DeviceClass:
        return DeviceClassFilter(re_expr)
    elif type == ElemType.Attribute:
        return AttributeFilter(re_expr)


class TaurusFilterPanelOld1(Qt.QWidget, taurus.qt.qtgui.base.TaurusBaseWidget):

    def __init__(self, parent=None, designMode=False):
        name = self.__class__.__name__
        self.call__init__wo_kw(Qt.QWidget, parent)
        self.call__init__(taurus.qt.qtgui.base.TaurusBaseWidget,
                          name, designMode=designMode)
        self.init()

    def init(self):
        l = Qt.QBoxLayout(Qt.QBoxLayout.TopToBottom)
        self.setLayout(l)
        self.addFilterHeader()
        self.insertFilterItem()
        l.addStretch(1)

    def addFilterHeader(self):
        label = Qt.QLabel("Type:")
        comboBox = Qt.QComboBox()
        comboBox.addItem(getElementTypeIcon(ElemType.Attribute),
                         "Attribute", ElemType.Attribute)
        comboBox.addItem(getElementTypeIcon(ElemType.Device),
                         "Device", ElemType.Device)
        comboBox.addItem(getElementTypeIcon(ElemType.DeviceClass),
                         "Device type", ElemType.DeviceClass)
        comboBox.addItem(getElementTypeIcon(ElemType.Server),
                         "Server", ElemType.Server)
        comboBox.addItem("Any")
        previewButton = Qt.QPushButton("Preview")
        previewButton.clicked.connect(self.onPreview)
        field = Qt.QWidget()
        l = Qt.QHBoxLayout()
        field.setLayout(l)
        l.addWidget(label)
        l.addWidget(comboBox)
        l.addWidget(previewButton)
        l.addStretch(1)
        self.layout().addWidget(field)

    def insertFilterItem(self, row=None):

        sl = self.layout()

        comboBox = Qt.QComboBox()
        self._fillComboBox(comboBox)
        comboBox.currentIndexChanged.connect(self.onFilterComboBoxItemSelected)

        edit = Qt.QLineEdit()

        addButton = Qt.QPushButton(Qt.QIcon(":/actions/list-add.svg"), "")
        addButton.clicked.connect(self.onAddFilterButtonClicked)

        removeButton = Qt.QPushButton(
            Qt.QIcon(":/actions/list-remove.svg"), "")
        removeButton.clicked.connect(self.onRemoveFilterButtonClicked)

        field = Qt.QWidget()
        l = Qt.QHBoxLayout()
        field.setLayout(l)
        l.addWidget(Qt.QLabel("Filter by"))
        l.addWidget(comboBox)
        l.addWidget(edit)
        l.addWidget(addButton)
        l.addWidget(removeButton)

        if row is None:
            sl.addWidget(field)
        else:
            sl.insertWidget(row, field)

    def _fillComboBox(self, comboBox):
        comboBox.addItem(getElementTypeIcon(ElemType.Attribute),
                         "Attribute", ElemType.Attribute)
        comboBox.addItem(getElementTypeIcon(ElemType.Device),
                         "Device", ElemType.Device)
        comboBox.addItem(getElementTypeIcon(ElemType.DeviceClass),
                         "Device type", ElemType.DeviceClass)
        comboBox.addItem(getElementTypeIcon(ElemType.Domain),
                         "Domain", ElemType.Domain)
        comboBox.addItem(getElementTypeIcon(ElemType.Family),
                         "Family", ElemType.Family)
        comboBox.addItem(getElementTypeIcon(ElemType.Member),
                         "Member", ElemType.Member)
        comboBox.addItem(getElementTypeIcon(ElemType.Server),
                         "Server", ElemType.Server)
        comboBox.addItem(getElementTypeIcon(ElemType.ServerName),
                         "Server Name", ElemType.ServerName)
        comboBox.addItem(getElementTypeIcon(ElemType.ServerInstance),
                         "Server Instance", ElemType.ServerInstance)

    def onFilterComboBoxItemSelected(self, index):
        pass

    def onAddFilterButtonClicked(self):
        button = self.sender()
        if button is None:
            return
        field = button.parent()
        index = self.layout().indexOf(field)
        self.insertFilterItem(index + 1)

    def onRemoveFilterButtonClicked(self):
        l = self.layout()
        # there is a header row, at least one filter row and a stretch at the
        # end, therefore, if there are only three rows, we don't allow to delete
        # the only existing filter
        if l.count() <= 3:
            return
        button = self.sender()
        if button is None:
            return
        field = button.parent()
        l.removeWidget(field)
        field.setParent(None)

    def onPreview(self):
        import trees
        model = self.getModel()
        dialog = Qt.QDialog()
        dialog.setModal(True)
        w = trees.TaurusTreeWidget(dialog, perspective=self.getHeaderType())
        w.setModel(model)
        w.setFilters(self.calculate())
        dialog.exec_()

    def getHeaderType(self):
        g_layout = self.layout()
        header = g_layout.itemAt(0).widget()
        headerCombo = header.layout().itemAt(1).widget()
        return Qt.from_qvariant(headerCombo.itemData(headerCombo.currentIndex()))

    def calculate(self):
        db = self.getModelObj()
        if db is None:
            return

        g_layout = self.layout()

        filters = []
        for i in xrange(1, g_layout.count() - 1):
            field_layout = g_layout.itemAt(i).widget().layout()
            comboBox = field_layout.itemAt(1).widget()
            edit = field_layout.itemAt(2).widget()

            type = Qt.from_qvariant(comboBox.itemData(comboBox.currentIndex()))
            expr = str(edit.text())
            f = getFilter(type, expr)
            filters.append(f)

        finalType = self.getHeaderType()
        filters.append(getFilter(finalType))
        return filters

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # TaurusBaseWidget overwriting
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def getModelClass(self):
        return TaurusAuthority

    #: This property holds the unique URI string representing the model name
    #: with which this widget will get its data from. The convention used for
    #: the string can be found :ref:`here <model-concept>`.
    #:
    #: In case the property :attr:`useParentModel` is set to True, the model
    #: text must start with a '/' followed by the attribute name.
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`TaurusBaseWidget.getModel`
    #:     * :meth:`TaurusBaseWidget.setModel`
    #:     * :meth:`TaurusBaseWidget.resetModel`
    #:
    #: .. seealso:: :ref:`model-concept`
    model = Qt.pyqtProperty("QString", taurus.qt.qtgui.base.TaurusBaseWidget.getModel,
                            taurus.qt.qtgui.base.TaurusBaseWidget.setModel,
                            taurus.qt.qtgui.base.TaurusBaseWidget.resetModel)


class TaurusFilterPanelOld2(Qt.QWidget, taurus.qt.qtgui.base.TaurusBaseWidget):

    def __init__(self, parent=None, designMode=False):
        name = self.__class__.__name__
        self.call__init__wo_kw(Qt.QWidget, parent)
        self.call__init__(taurus.qt.qtgui.base.TaurusBaseWidget,
                          name, designMode=designMode)
        self.init()

    def init(self):
        l = Qt.QGridLayout()
        l.setContentsMargins(0, 0, 0, 0)
        self.setLayout(l)

        comboBox = Qt.QComboBox()
        comboBox.addItem(getElementTypeIcon(ElemType.Attribute),
                         "Attribute", ElemType.Attribute)
        comboBox.addItem(getElementTypeIcon(ElemType.Device),
                         "Device", ElemType.Device)
        comboBox.addItem(getElementTypeIcon(ElemType.DeviceClass),
                         "Device type", ElemType.DeviceClass)
        comboBox.addItem(getElementTypeIcon(ElemType.Server),
                         "Server", ElemType.Server)

        l.addWidget(Qt.QLabel("Filter for:"), 0, 0)
        l.addWidget(comboBox, 0, 1)

        import trees
        self._deviceEdit = Qt.QComboBox()
        self._deviceEdit.setEditable(True)
        self._deviceEdit.setMaxVisibleItems(10)
        self._deviceEdit.setInsertPolicy(Qt.QComboBox.InsertAtTop)
        self._deviceDomainEdit = Qt.QLineEdit()
        self._deviceFamilyEdit = Qt.QLineEdit()
        self._deviceMemberEdit = Qt.QLineEdit()
        self._deviceClass = Qt.QLineEdit()
        self._serverEdit = Qt.QLineEdit()
        self._serverNameEdit = Qt.QLineEdit()
        self._serverInstanceEdit = Qt.QLineEdit()
        self._attributeEdit = Qt.QLineEdit()

        lbl = Qt.QLabel("Device type:")
        l.addWidget(lbl, 1, 0)
        l.setAlignment(lbl, Qt.Qt.AlignRight)
        l.addWidget(self._deviceClass, 1, 1)
        l.addWidget(Qt.QLabel("Device:"), 2, 0)
        l.addWidget(self._deviceEdit, 2, 1)
        l.addWidget(Qt.QLabel("Device domain:"), 3, 0)
        l.addWidget(self._deviceDomainEdit, 3, 1)
        l.addWidget(Qt.QLabel("Device family:"), 4, 0)
        l.addWidget(self._deviceFamilyEdit, 4, 1)
        l.addWidget(Qt.QLabel("Device member:"), 5, 0)
        l.addWidget(self._deviceMemberEdit, 5, 1)
        l.addWidget(Qt.QLabel("Server:"), 6, 0)
        l.addWidget(self._serverEdit, 6, 1)
        l.addWidget(Qt.QLabel("Server name:"), 7, 0)
        l.addWidget(self._serverNameEdit, 7, 1)
        l.addWidget(Qt.QLabel("Server instance:"), 8, 0)
        l.addWidget(self._serverInstanceEdit, 8, 1)
        l.addWidget(Qt.QLabel("Attribute:"), 9, 0)
        l.addWidget(self._attributeEdit, 9, 1)

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # TaurusBaseWidget overwriting
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def getModelClass(self):
        return TaurusAuthority

    def setModel(self, m):
        taurus.qt.qtgui.base.TaurusBaseWidget.setModel(self, m)
        db = self.getModelObj()
        #model = self._deviceEdit.model()
        # if model is None: return
        # model.setDataSource(db)
        self._deviceEdit.clear()
        if db is not None:
            deviceNames = db.cache().getDeviceNames()
            deviceNames.sort()
            #icon = taurus.core.icons.getElementTypeIcon(ElemType.Device)
            self._deviceEdit.addItems(deviceNames)

    #: This property holds the unique URI string representing the model name
    #: with which this widget will get its data from. The convention used for
    #: the string can be found :ref:`here <model-concept>`.
    #:
    #: In case the property :attr:`useParentModel` is set to True, the model
    #: text must start with a '/' followed by the attribute name.
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`TaurusBaseWidget.getModel`
    #:     * :meth:`TaurusBaseWidget.setModel`
    #:     * :meth:`TaurusBaseWidget.resetModel`
    #:
    #: .. seealso:: :ref:`model-concept`
    model = Qt.pyqtProperty("QString", taurus.qt.qtgui.base.TaurusBaseWidget.getModel,
                            taurus.qt.qtgui.base.TaurusBaseWidget.setModel,
                            taurus.qt.qtgui.base.TaurusBaseWidget.resetModel)


class _MessageWidget(Qt.QWidget):

    def __init__(self, parent=None, pixmap=None):
        Qt.QWidget.__init__(self, parent)
        l = Qt.QHBoxLayout()
        self.setLayout(l)
        self._icon = Qt.QLabel()
        if pixmap is None:
            pixmap = Qt.QIcon.fromTheme("dialog-warning").pixmap(16, 16)
        self._icon.setPixmap(pixmap)
        self._label = Qt.QLabel()
        l.addWidget(self._icon)
        l.addWidget(self._label)

    def setText(self, text):
        self._label.setText(text)


from taurus.external.qt.uic import loadUi


class TaurusFilterPanel(Qt.QWidget, taurus.qt.qtgui.base.TaurusBaseWidget):

    _Items = "server", "serverName", "serverInstance", \
             "deviceName", "deviceType", "deviceDomain", "deviceFamily", "deviceMember", \
             "attribute"

    def __init__(self, parent=None, designMode=False):
        name = self.__class__.__name__
        self.call__init__wo_kw(Qt.QWidget, parent)
        self.call__init__(taurus.qt.qtgui.base.TaurusBaseWidget,
                          name, designMode=designMode)
        self.init()

    def init(self):
        l = Qt.QVBoxLayout()
        self.setLayout(l)

        panel = self._mainPanel = Qt.QWidget()
        l.addWidget(panel, 1)
        this_dir = os.path.dirname(os.path.abspath(__file__))
        ui_filename = os.path.join(this_dir, 'ui', 'TaurusFilterPanel.ui')
        self.ui = ui = loadUi(ui_filename, baseinstance=panel)

        comboBox = ui.filterTypeCombo
        comboBox.addItem(getElementTypeIcon(ElemType.Attribute),
                         "Attribute", ElemType.Attribute)
        comboBox.addItem(getElementTypeIcon(ElemType.Device),
                         "Device", ElemType.Device)
        comboBox.addItem(getElementTypeIcon(ElemType.DeviceClass),
                         "Device type", ElemType.DeviceClass)
        comboBox.addItem(getElementTypeIcon(ElemType.Server),
                         "Server", ElemType.Server)

        clickedSig = Qt.SIGNAL("clicked()")
        idxChangedSig = Qt.SIGNAL("currentIndexChanged(int)")
        ui.serverNameCombo.currentIndexChanged.connect(
                           self._updateServerInstanceCombo)
        ui.deviceDomainCombo.currentIndexChanged.connect(
                           self._updateDeviceFamilyCombo)
        ui.deviceFamilyCombo.currentIndexChanged.connect(
                           self._updateDeviceMemberCombo)

        class clearSelection(object):

            def __init__(self, cb):
                self._cb = cb

            def __call__(self):
                self._cb.setCurrentIndex(-1)

        clear_icon = Qt.QIcon.fromTheme("edit-clear")
        for combo, clearButton in zip(self.combos(), self.clearButtons()):
            combo.currentIndexChanged.connect(self._updateStatusBar)
            clearButton.clicked.connect(clearSelection(combo))
            clearButton.setIcon(clear_icon)

        sb = self._statusbar = Qt.QStatusBar()
        sb.setSizeGripEnabled(False)
        l.addWidget(sb)
        sbWarningMsg = self._sbWarningMsg = _MessageWidget()
        sbWarningMsg.setVisible(False)
        sb.addWidget(sbWarningMsg)

    def combos(self):
        if not hasattr(self, "_combos"):
            f = self.ui
            self._combos = [getattr(f, name + "Combo") for name in self._Items]
        return self._combos

    def clearButtons(self):
        if not hasattr(self, "_clearButtons"):
            f = self.ui
            self._clearButtons = [getattr(f, name + "ClearButton")
                                  for name in self._Items]
        return self._clearButtons

    def _db_cache(self):
        db = self.getModelObj()
        if db is None:
            return
        return db.cache()

    def _updateStatusBar(self, index=None):
        form = self.ui
        server = str(form.serverCombo.currentText())
        serverName = str(form.serverNameCombo.currentText())
        serverInstance = str(form.serverInstanceCombo.currentText())

        msg = self._sbWarningMsg
        msg.setVisible(False)

        if server and (serverName or serverInstance):
            sb = self._statusbar
            s = "Specifying name filter and type/instance filters at the same " \
                "time may result in an empty filter"
            msg.setVisible(True)
            msg.setText(s)

        deviceName = str(form.deviceNameCombo.currentText())
        deviceDomain = str(form.deviceDomainCombo.currentText())
        deviceFamily = str(form.deviceFamilyCombo.currentText())
        deviceMember = str(form.deviceMemberCombo.currentText())

        if deviceName and (deviceDomain or deviceFamily or deviceMember):
            sb = self._statusbar
            s = "Specifying name filter and domain/family/member filters at the same " \
                "time may result in an empty filter"
            msg.setVisible(True)
            msg.setText(s)

    def _updateServerCombo(self, index=None):
        combo = self.ui.serverCombo
        combo.clear()
        db_cache = self._db_cache()
        if db_cache is None:
            return
        servers = db_cache.servers()
        icon = getElementTypeIcon(ElemType.Server)
        for serverName in sorted(servers):
            serverInfo = servers[serverName]
            combo.addItem(icon, serverName, serverInfo)
        combo.setCurrentIndex(-1)

    def _updateServerNameCombo(self, index=None):
        combo = self.ui.serverNameCombo
        combo.clear()
        db_cache = self._db_cache()
        if db_cache is None:
            return
        servers = db_cache.servers()
        serverNames = []
        for server in servers.values():
            name = server.serverName()
            if name not in serverNames:
                serverNames.append(name)
        serverNames.sort()
        icon = getElementTypeIcon(ElemType.ServerName)
        for serverName in serverNames:
            combo.addItem(icon, serverName)
        combo.setCurrentIndex(-1)

    def _updateServerInstanceCombo(self, index=None):
        combo = self.ui.serverInstanceCombo
        combo.clear()
        db_cache = self._db_cache()
        if db_cache is None:
            return
        if index is None or index == -1:
            return
        serverName = str(self.sender().currentText())
        servers = db_cache.servers()
        serverInstances = []
        for server in servers.values():
            if server.serverName() == serverName:
                serverInstances.append(server.serverInstance())
        serverInstances.sort()
        icon = getElementTypeIcon(ElemType.ServerInstance)
        for serverInstance in serverInstances:
            combo.addItem(icon, serverInstance)
        combo.setCurrentIndex(-1)

    def _updateDeviceTypeCombo(self, index=None):
        combo = self.ui.deviceTypeCombo
        combo.clear()
        db_cache = self._db_cache()
        if db_cache is None:
            return
        deviceKlasses = db_cache.klasses()
        icon = getElementTypeIcon(ElemType.DeviceClass)
        for klassName in sorted(deviceKlasses):
            klassInfo = deviceKlasses[klassName]
            combo.addItem(icon, klassName, klassInfo)
        combo.setCurrentIndex(-1)

    def _updateDeviceNameCombo(self, index=None):
        combo = self.ui.deviceNameCombo
        combo.clear()
        db_cache = self._db_cache()
        if db_cache is None:
            return
        devices = db_cache.devices()
        icon = getElementTypeIcon(ElemType.Device)
        for deviceName in sorted(devices):
            deviceInfo = devices[deviceName]
            combo.addItem(icon, deviceName, deviceInfo)
        combo.setCurrentIndex(-1)

    def _updateDeviceDomainCombo(self, index=None):
        combo = self.ui.deviceDomainCombo
        combo.clear()
        db_cache = self._db_cache()
        if db_cache is None:
            return
        domains = db_cache.getDeviceDomainNames()
        domains.sort()
        icon = getElementTypeIcon(ElemType.Domain)
        for domain in domains:
            combo.addItem(icon, domain)
        combo.setCurrentIndex(-1)

    def _updateDeviceFamilyCombo(self, index=None):
        combo = self.ui.deviceFamilyCombo
        combo.clear()
        db_cache = self._db_cache()
        if db_cache is None:
            return

        deviceDomain = str(self.ui.deviceDomainCombo.currentText())
        if deviceDomain == "":
            return
        families = db_cache.getDeviceFamilyNames(deviceDomain)
        families.sort()
        icon = getElementTypeIcon(ElemType.Family)
        for family in families:
            combo.addItem(icon, family)
        combo.setCurrentIndex(-1)

    def _updateDeviceMemberCombo(self, index=None):
        combo = self.ui.deviceMemberCombo
        combo.clear()
        db_cache = self._db_cache()
        if db_cache is None:
            return

        deviceDomain = str(self.ui.deviceDomainCombo.currentText())
        if deviceDomain == "":
            return
        deviceFamily = str(self.ui.deviceFamilyCombo.currentText())
        if deviceFamily == "":
            return
        members = db_cache.getDeviceMemberNames(deviceDomain, deviceFamily)
        members.sort()
        icon = getElementTypeIcon(ElemType.Member)
        for member in members:
            combo.addItem(icon, member)
        combo.setCurrentIndex(-1)

    def _updateAttributeCombo(self, index=None):
        combo = self.ui.attributeCombo
        combo.clear()
        db_cache = self._db_cache()
        if db_cache is None:
            return

    def _fillItems(self):
        self._updateServerCombo()
        self._updateServerNameCombo()
        self._updateServerInstanceCombo()
        self._updateDeviceTypeCombo()
        self._updateDeviceNameCombo()
        self._updateDeviceDomainCombo()
        self._updateDeviceFamilyCombo()
        self._updateDeviceMemberCombo()
        self._updateAttributeCombo()

    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
    # TaurusBaseWidget overwriting
    #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

    def getModelClass(self):
        return TaurusAuthority

    def setModel(self, m):
        taurus.qt.qtgui.base.TaurusBaseWidget.setModel(self, m)
        db = self.getModelObj()
        #model = self._deviceEdit.model()
        # if model is None: return
        # model.setDataSource(db)
        self.ui.deviceNameCombo.clear()
        self._fillItems()

    #: This property holds the unique URI string representing the model name
    #: with which this widget will get its data from. The convention used for
    #: the string can be found :ref:`here <model-concept>`.
    #:
    #: In case the property :attr:`useParentModel` is set to True, the model
    #: text must start with a '/' followed by the attribute name.
    #:
    #: **Access functions:**
    #:
    #:     * :meth:`TaurusBaseWidget.getModel`
    #:     * :meth:`TaurusBaseWidget.setModel`
    #:     * :meth:`TaurusBaseWidget.resetModel`
    #:
    #: .. seealso:: :ref:`model-concept`
    model = Qt.pyqtProperty("QString", taurus.qt.qtgui.base.TaurusBaseWidget.getModel,
                            taurus.qt.qtgui.base.TaurusBaseWidget.setModel,
                            taurus.qt.qtgui.base.TaurusBaseWidget.resetModel)


def main():
    from taurus.qt.qtgui.application import TaurusApplication
    from taurus.core.util import argparse
    import sys

    parser = argparse.get_taurus_parser()
    parser.usage = "%prog [options] [hostname]"

    app = TaurusApplication(cmd_line_parser=parser)
    args = app.get_command_line_args()

    if len(args) > 0:
        host = args[0]
    else:
        host = taurus.Authority().getNormalName()

    w = TaurusFilterPanel()
    w.setWindowIcon(Qt.QIcon(":/actions/system-shutdown.svg"))
    w.setWindowTitle("A Taurus Filter Example")
    w.setModel(host)
    w.show()

    sys.exit(app.exec_())

if __name__ == "__main__":
    main()
